@@ -1,14 +1,17 @@
 import asyncio
 import logging
 import os
+import re
 from collections import defaultdict
 from collections.abc import Callable
 from typing import Any
-import re
+
+from httpx import HTTPStatusError
 
 from mcp_scan.models import Issue, ScanError, ScanPathResult, ServerScanResult
-from mcp_scan.well_known_clients import get_client_from_path, get_builtin_tools
+from mcp_scan.well_known_clients import get_builtin_tools, get_client_from_path
 
+from .direct_scanner import direct_scan, is_direct_scan
 from .mcp_client import check_server_with_timeout, scan_mcp_config_file
 from .Storage import Storage
 from .verify_api import analyze_scan_path
@@ -53,19 +56,22 @@ class MCPScanner:
     def __init__(
         self,
         files: list[str] | None = None,
-        base_url: str = "https://mcp.invariantlabs.ai/",
+        analysis_url: str = "https://mcp.invariantlabs.ai/api/v1/public/mcp-analysis",
         checks_per_server: int = 1,
         storage_file: str = "~/.mcp-scan",
         server_timeout: int = 10,
         suppress_mcpserver_io: bool = True,
         opt_out: bool = False,
         include_built_in: bool = False,
+        verbose: bool = False,
+        additional_headers: dict = {},
         **kwargs: Any,
     ):
         logger.info("Initializing MCPScanner")
         self.paths = files or []
         logger.debug("Paths to scan: %s", self.paths)
-        self.base_url = base_url
+        self.analysis_url = analysis_url
+        self.additional_headers = additional_headers
         self.checks_per_server = checks_per_server
         self.storage_file_path = os.path.expanduser(storage_file)
         logger.debug("Storage file path: %s", self.storage_file_path)
@@ -75,6 +81,7 @@ class MCPScanner:
         self.context_manager = None
         self.opt_out_of_identity = opt_out
         self.include_built_in = include_built_in
+        self.verbose = verbose
         logger.debug(
             "MCPScanner initialized with timeout: %d, checks_per_server: %d", server_timeout, checks_per_server
         )
@@ -114,7 +121,10 @@ class MCPScanner:
         logger.info("Getting servers from path: %s", path)
         result = ScanPathResult(path=path)
         try:
-            servers = (await scan_mcp_config_file(path)).get_servers()
+            if not os.path.exists(path) and is_direct_scan(path):
+                servers = (await direct_scan(path)).get_servers()
+            else:
+                servers = (await scan_mcp_config_file(path)).get_servers()
             logger.debug("Found %d servers in path: %s", len(servers), path)
             result.servers = [
                 ServerScanResult(name=server_name, server=server) for server_name, server in servers.items()
@@ -122,16 +132,19 @@ class MCPScanner:
         except FileNotFoundError as e:
             error_msg = "file does not exist"
             logger.exception("%s: %s", error_msg, path)
-            result.error = ScanError(message=error_msg, exception=e)
+            # This is a non failing error, so we set is_failure to False.
+            result.error = ScanError(message=error_msg, exception=e, is_failure=False)
         except Exception as e:
             error_msg = "could not parse file"
             logger.exception("%s: %s", error_msg, path)
-            result.error = ScanError(message=error_msg, exception=e)
+            result.error = ScanError(message=error_msg, exception=e, is_failure=True)
         return result
 
     def check_server_changed(self, path_result: ScanPathResult) -> list[Issue]:
         logger.debug("Checking server changed: %s", path_result.path)
         issues: list[Issue] = []
+        if path_result.servers is None:
+            return issues
         for server_idx, server in enumerate(path_result.servers):
             logger.debug(
                 "Checking for changes in server %d/%d: %s", server_idx + 1, len(path_result.servers), server.name
@@ -152,6 +165,8 @@ class MCPScanner:
     def check_whitelist(self, path_result: ScanPathResult) -> list[Issue]:
         logger.debug("Checking whitelist for path: %s", path_result.path)
         issues: list[Issue] = []
+        if path_result.servers is None:
+            return issues
         for server_idx, server in enumerate(path_result.servers):
             for entity_idx, entity in enumerate(server.entities):
                 if self.storage_file.is_whitelisted(entity):
@@ -182,10 +197,14 @@ class MCPScanner:
                 len(result.signature.resource_templates),
                 len(result.signature.tools),
             )
+        except HTTPStatusError as e:
+            error_msg = "server returned HTTP status code"
+            logger.exception("%s: %s", error_msg, server.name)
+            result.error = ScanError(message=error_msg, exception=e, is_failure=True)
         except Exception as e:
             error_msg = "could not start server"
             logger.exception("%s: %s", error_msg, server.name)
-            result.error = ScanError(message=error_msg, exception=e)
+            result.error = ScanError(message=error_msg, exception=e, is_failure=True)
         await self.emit("server_scanned", result)
         return result
     
@@ -195,15 +214,16 @@ class MCPScanner:
         logger.info("Scanning path: %s, inspect_only: %s", path, inspect_only)
         path_result = await self.get_servers_from_path(path)
 
-        for i, server in enumerate(path_result.servers):
-            if server.server.type == "stdio":
-                full_command = server.server.command + " " + " ".join(server.server.args or [])
-                # check if pattern is contained in full_command
-                if re.search(r"mcp[-_]scan.*mcp-server", full_command):
-                    logger.info("Skipping scan of server %d/%d: %s", i + 1, len(path_result.servers), server.name)
-                    continue
-            logger.debug("Scanning server %d/%d: %s", i + 1, len(path_result.servers), server.name)
-            path_result.servers[i] = await self.scan_server(server)
+        if path_result.servers is not None:
+            for i, server in enumerate(path_result.servers):
+                if server.server.type == "stdio":
+                    full_command = server.server.command + " " + " ".join(server.server.args or [])
+                    # check if pattern is contained in full_command
+                    if re.search(r"mcp[-_]scan.*mcp-server", full_command):
+                        logger.info("Skipping scan of server %d/%d: %s", i + 1, len(path_result.servers), server.name)
+                        continue
+                logger.debug("Scanning server %d/%d: %s", i + 1, len(path_result.servers), server.name)
+                path_result.servers[i] = await self.scan_server(server)
 
         # add built-in tools
         if self.include_built_in:
@@ -220,7 +240,7 @@ class MCPScanner:
         path_result.issues += self.check_server_changed(path_result)
         logger.debug(f"Verifying server path: {path_result.path}, {path_result.path is None}")
         path_result = await analyze_scan_path(
-            path_result, base_url=self.base_url, opt_out_of_identity=self.opt_out_of_identity
+            path_result, analysis_url=self.analysis_url, additional_headers=self.additional_headers, opt_out_of_identity=self.opt_out_of_identity, verbose=self.verbose
         )
         await self.emit("path_scanned", path_result)
         return path_result
