@@ -1,5 +1,6 @@
 from .MCPScanner import MCPScanner
 from .version import version_info
+from .lib import scan, inspect
 
-__all__ = ["MCPScanner"]
+__all__ = ["MCPScanner", "scan", "inspect"]
 __version__ = version_info
