@@ -20,6 +20,7 @@ from .well_known_clients import WELL_KNOWN_MCP_PATHS, client_shorthands_to_paths
 from .printer import print_scan_result
 from .Storage import Storage
 from .version import version_info
+from .utils import parse_headers
 
 # Configure logging to suppress all output by default
 logging.getLogger().setLevel(logging.CRITICAL + 1)  # Higher than any standard level
@@ -37,7 +38,7 @@ def setup_logging(verbose=False, log_to_stderr=False):
             root_logger.removeHandler(hdlr)
         if log_to_stderr:
             # stderr logging
-            stderr_console = rich.Console(stderr=True)
+            stderr_console = rich.console.Console(stderr=True)
             logging.basicConfig(
                 format="%(message)s",
                 datefmt="[%X]",
@@ -79,6 +80,62 @@ def str2bool(v: str) -> bool:
     return v.lower() in ("true", "1", "t", "y", "yes")
 
 
+def parse_control_servers(argv):
+    """
+    Parse control server arguments from sys.argv.
+    Returns a list of control server configurations, where each config is a dict with:
+    - url: the control server URL
+    - headers: list of additional headers
+    - identifier: the control identifier (or None)
+    - opt_out: boolean indicating if opt-out is enabled
+    """
+    control_servers = []
+    current_server = None
+    
+    i = 0
+    while i < len(argv):
+        arg = argv[i]
+        
+        if arg == "--control-server":
+            # Save previous server if exists
+            if current_server is not None:
+                control_servers.append(current_server)
+            
+            # Start new server config
+            if i + 1 < len(argv) and not argv[i + 1].startswith("--"):
+                current_server = {
+                    "url": argv[i + 1],
+                    "headers": [],
+                    "identifier": None,
+                    "opt_out": False,
+                }
+                i += 1  # Skip the URL value
+            else:
+                current_server = None
+        
+        elif current_server is not None:
+            if arg == "--control-server-H":
+                if i + 1 < len(argv) and not argv[i + 1].startswith("--"):
+                    current_server["headers"].append(argv[i + 1])
+                    i += 1
+            
+            elif arg == "--control-identifier":
+                if i + 1 < len(argv) and not argv[i + 1].startswith("--"):
+                    current_server["identifier"] = argv[i + 1]
+                    i += 1
+            
+            elif arg == "--opt-out":
+                current_server["opt_out"] = True
+        
+        i += 1
+    
+    # Don't forget the last server
+    if current_server is not None:
+        control_servers.append(current_server)
+    
+    return control_servers
+
+
 def add_common_arguments(parser):
     """Add arguments that are common to multiple commands."""
     parser.add_argument(
@@ -89,12 +146,18 @@ def add_common_arguments(parser):
         metavar="FILE",
     )
     parser.add_argument(
-        "--base-url",
+        "--analysis-url",
         type=str,
-        default="https://mcp.invariantlabs.ai/",
-        help="Base URL for the verification server",
+        default="https://mcp.invariantlabs.ai/api/v1/public/mcp-analysis",
+        help="URL endpoint for the verification server",
         metavar="URL",
     )
+    parser.add_argument(
+        "--verification-H",
+        action="append",
+        help="Additional headers for the verification server",
+    )
+
     parser.add_argument(
         "--verbose",
         default=False,
@@ -208,24 +271,24 @@ def add_scan_arguments(scan_parser):
     )
     scan_parser.add_argument(
         "--control-server",
-        default=False,
-        help="Upload the scan results to the provided control server URL (default: Do not upload)",
+        action="append",
+        help="Upload the scan results to the provided control server URL. Can be specified multiple times for multiple control servers.",
     )
     scan_parser.add_argument(
-        "--push-key",
-        default=False,
-        help="When uploading the scan results to the provided control server URL, pass the push key (default: Do not upload)",
+        "--control-server-H",
+        action="append",
+        help="Additional headers for the preceding control server",
     )
     scan_parser.add_argument(
-        "--email",
-        default=None,
-        help="When uploading the scan results to the provided control server URL, pass the email.",
+        "--control-identifier",
+        action="append",
+        help="Non-anonymous identifier used to identify the user to the preceding control server, e.g. email or serial number",
     )
     scan_parser.add_argument(
         "--opt-out",
-        default=False,
-        action="store_true",
-        help="Opts out of sending unique a unique user identifier with every scan.",
+        action="append_const",
+        const=True,
+        help="Opts out of sending a unique user identifier with every scan to the preceding control server.",
     )
     scan_parser.add_argument(
         "--include-built-in",
@@ -233,7 +296,6 @@ def add_scan_arguments(scan_parser):
         action="store_true",
         help="Also include built-in IDE tools.",
     )
-   
 
 
 def add_uninstall_arguments(parser):
@@ -296,6 +358,11 @@ def main():
             f"  {program_name} --verbose           # Enable detailed logging output\n"
             f"  {program_name} --print-errors      # Show error details and tracebacks\n"
             f"  {program_name} --json              # Output results in JSON format\n"
+            f"  # Multiple control servers with individual options:\n"
+            f'  {program_name} --control-server https://server1.com --control-server-H "Auth: token1" \\\n'
+            f'    --control-identifier user@example.com --opt-out \\\n'
+            f'    --control-server https://server2.com --control-server-H "Auth: token2" \\\n'
+            f'    --control-identifier serial-123\n'
         ),
     )
 
@@ -445,16 +512,24 @@ def main():
     add_common_arguments(proxy_parser)
     add_server_arguments(proxy_parser)
     add_install_arguments(proxy_parser)
+    
 
     # Parse arguments (default to 'scan' if no command provided)
     if len(sys.argv) == 1 or sys.argv[1] not in subparsers.choices:
-        if sys.argv[1] != '--help':
+        if not (len(sys.argv) == 2 and sys.argv[1] == '--help'):
             sys.argv.insert(1, "scan")
+    
+    # Parse control servers before argparse to preserve their grouping
+    control_servers = parse_control_servers(sys.argv)
+    
     args = parser.parse_args()
 
     # postprocess the files argument (if shorthands are used)
     if hasattr(args, "files") and args.files is None:
         args.files = client_shorthands_to_paths(args.files)
+    
+    # Attach parsed control servers to args
+    args.control_servers = control_servers
 
     # Display version banner
     if not ((hasattr(args, "json") and args.json) or (args.command == "mcp-server")):
@@ -497,7 +572,7 @@ def main():
     setup_logging(do_log, log_to_stderr=(args.command != "mcp-server"))
 
     # Handle commands
-    if args.command == "help" or (args.command is None and args.help):
+    if args.command == "help" or (args.command is None and hasattr(args, "help") and args.help):
         parser.print_help()
         sys.exit(0)
     elif args.command == "whitelist":
@@ -510,12 +585,7 @@ def main():
             sf.print_whitelist()
             sys.exit(0)
         elif all(x is not None for x in [args.type, args.name, args.hash]):
-            sf.add_to_whitelist(
-                args.type,
-                args.name,
-                args.hash,
-                base_url=args.base_url if not args.local_only else None,
-            )
+            sf.add_to_whitelist(args.type, args.name, args.hash)
             sf.print_whitelist()
             sys.exit(0)
         else:
@@ -559,8 +629,7 @@ def main():
 
 
 async def run_scan_inspect(mode="scan", args=None):
-    async with MCPScanner(**vars(args)) as scanner:
-        # scanner.hook('path_scanned', print_path_scanned)
+    async with MCPScanner(additional_headers=parse_headers(args.verification_H), **vars(args)) as scanner:
         if mode == "scan":
             result = await scanner.scan()
         elif mode == "inspect":
@@ -568,16 +637,17 @@ async def run_scan_inspect(mode="scan", args=None):
         else:
             raise ValueError(f"Unknown mode: {mode}, expected 'scan' or 'inspect'")
 
-    # upload scan result to control server if specified
-    if (
-        hasattr(args, "control_server")
-        and args.control_server
-        and hasattr(args, "push_key")
-        and args.push_key
-        and hasattr(args, "email")
-        and hasattr(args, "opt_out")
-    ):
-        await upload(result, args.control_server, args.push_key, args.email, args.opt_out)
+    # upload scan result to control servers if specified
+    if hasattr(args, "control_servers") and args.control_servers:
+        for server_config in args.control_servers:
+            await upload(
+                result,
+                server_config["url"],
+                server_config["identifier"],
+                server_config["opt_out"],
+                verbose=hasattr(args, "verbose") and args.verbose,
+                additional_headers=parse_headers(server_config["headers"])
+            )
     return result
 
 async def print_scan_inspect(mode="scan", args=None):
