@@ -3,9 +3,9 @@ from datetime import datetime
 from hashlib import md5
 from itertools import chain
 from typing import Any, Literal, TypeAlias
-
+import re
 from mcp.types import InitializeResult, Prompt, Resource, Tool, ResourceTemplate, Completion
-from pydantic import BaseModel, ConfigDict, Field, RootModel, field_serializer, field_validator
+from pydantic import BaseModel, ConfigDict, Field, RootModel, field_serializer, field_validator, model_validator, ValidationError
 
 logger = logging.getLogger(__name__)
 
@@ -60,21 +60,21 @@ class ScannedEntity(BaseModel):
             raise ValueError(f"Unrecognized datetime format: {value}") from e
 
 
+class ScalarToolLabels(BaseModel):
+    is_public_sink: int | float
+    destructive: int | float
+    untrusted_content: int | float
+    private_data: int | float
+
+
 ScannedEntities = RootModel[dict[str, ScannedEntity]]
 
 
-class SSEServer(BaseModel):
+class RemoteServer(BaseModel):
     model_config = ConfigDict()
     url: str
-    type: Literal["sse"] | None = "sse"
-    headers: dict[str, str] = {}
-
-
-class StreamableHTTPServer(BaseModel):
-    model_config = ConfigDict()
-    url: str
-    type: Literal["http"] | None = "http"
-    headers: dict[str, str] = {}
+    type: Literal["sse", "http"] | None = None
+    headers: dict[str, str] = Field(default_factory=dict)
 
 
 class StdioServer(BaseModel):
@@ -84,23 +84,39 @@ class StdioServer(BaseModel):
     type: Literal["stdio"] | None = "stdio"
     env: dict[str, str] | None = None
 
+class StaticToolsServer(BaseModel):
+    """A server with a static set of tools (e.g. if not scanning a MCP configuration but the set of tools directly)."""
+    model_config = ConfigDict()
+    name: str
+    signature: list[Tool]
+    type: Literal["tools"] | None = "tools"
 
 class MCPConfig(BaseModel):
-    def get_servers(self) -> dict[str, SSEServer | StdioServer | StreamableHTTPServer]:
+    def get_servers(self) -> dict[str, StdioServer |  RemoteServer]:
         raise NotImplementedError("Subclasses must implement this method")
 
-    def set_servers(self, servers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]) -> None:
+    def set_servers(self, servers: dict[str, StdioServer | RemoteServer]) -> None:
         raise NotImplementedError("Subclasses must implement this method")
 
+class StaticToolsConfig(MCPConfig):
+    model_config = ConfigDict()
+    signature: dict[str, StaticToolsServer]
+
+    def get_servers(self) -> dict[str, StdioServer | RemoteServer]:
+        return {server.name: server for server in self.signature.values()}
+
+    def set_servers(self, servers: dict[str, StdioServer | RemoteServer]) -> None:
+        raise NotImplementedError("StaticToolsConfig does not support setting servers")
+
 
 class ClaudeConfigFile(MCPConfig):
     model_config = ConfigDict()
-    mcpServers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]
+    mcpServers: dict[str, StdioServer | RemoteServer]
 
-    def get_servers(self) -> dict[str, SSEServer | StdioServer | StreamableHTTPServer]:
+    def get_servers(self) -> dict[str, StdioServer | RemoteServer]:
         return self.mcpServers
 
-    def set_servers(self, servers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]) -> None:
+    def set_servers(self, servers: dict[str, StdioServer | RemoteServer]) -> None:
         self.mcpServers = servers
 
 
@@ -108,12 +124,12 @@ class VSCodeMCPConfig(MCPConfig):
     # see https://code.visualstudio.com/docs/copilot/chat/mcp-servers
     model_config = ConfigDict()
     inputs: list[Any] | None = None
-    servers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]
+    servers: dict[str, StdioServer | RemoteServer]
 
-    def get_servers(self) -> dict[str, SSEServer | StdioServer | StreamableHTTPServer]:
+    def get_servers(self) -> dict[str, StdioServer | RemoteServer]:
         return self.servers
 
-    def set_servers(self, servers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]) -> None:
+    def set_servers(self, servers: dict[str, StdioServer | RemoteServer]) -> None:
         self.servers = servers
 
 
@@ -121,10 +137,10 @@ class VSCodeConfigFile(MCPConfig):
     model_config = ConfigDict()
     mcp: VSCodeMCPConfig
 
-    def get_servers(self) -> dict[str, SSEServer | StdioServer | StreamableHTTPServer]:
+    def get_servers(self) -> dict[str, StdioServer | RemoteServer]:
         return self.mcp.servers
 
-    def set_servers(self, servers: dict[str, SSEServer | StdioServer | StreamableHTTPServer]) -> None:
+    def set_servers(self, servers: dict[str, StdioServer | RemoteServer]) -> None:
         self.mcp.servers = servers
 
 
@@ -132,6 +148,7 @@ class ScanError(BaseModel):
     model_config = ConfigDict(arbitrary_types_allowed=True)
     message: str | None = None
     exception: Exception | None = None
+    is_failure: bool = True
 
     @field_serializer("exception")
     def serialize_exception(self, exception: Exception | None, _info) -> str | None:
@@ -149,6 +166,7 @@ class ScanError(BaseModel):
         return ScanError(
             message=self.message,
             exception=self.exception,
+            is_failure=self.is_failure,
         )
 
 
@@ -184,7 +202,7 @@ class VerifyServerRequest(RootModel[list[ServerSignature | None]]):
 class ServerScanResult(BaseModel):
     model_config = ConfigDict()
     name: str | None = None
-    server: SSEServer | StdioServer | StreamableHTTPServer
+    server: StdioServer | RemoteServer | StaticToolsServer
     signature: ServerSignature | None = None
     error: ScanError | None = None
 
@@ -215,14 +233,18 @@ class ServerScanResult(BaseModel):
 
 class ScanPathResult(BaseModel):
     model_config = ConfigDict()
+    client: str | None = None
     path: str
-    servers: list[ServerScanResult] = Field(default_factory=list)
+    # servers is None if the MCP configuration file was missing or unparseable
+    # which prevented server discovery.
+    servers: list[ServerScanResult] | None = None
     issues: list[Issue] = Field(default_factory=list)
+    labels: list[list[ScalarToolLabels]] = Field(default_factory=list)
     error: ScanError | None = None
 
     @property
     def entities(self) -> list[Entity]:
-        return list(chain.from_iterable(server.entities for server in self.servers))
+        return list(chain.from_iterable(server.entities for server in self.servers)) if self.servers else []
 
     def clone(self) -> "ScanPathResult":
         """
@@ -231,8 +253,10 @@ class ScanPathResult(BaseModel):
         """
         output = ScanPathResult(
             path=self.path,
-            servers=[server.clone() for server in self.servers],
+            client=self.client,
+            servers=[server.clone() for server in self.servers] if self.servers else None,
             issues=[issue.model_copy(deep=True) for issue in self.issues],
+            labels=[[label.model_copy(deep=True) for label in labels] for labels in self.labels],
             error=self.error.clone() if self.error else None,
         )
         return output
@@ -241,7 +265,7 @@ class ScanPathResult(BaseModel):
 class ScanUserInfo(BaseModel):
     hostname: str | None = None
     username: str | None = None
-    email: str | None = None
+    identifier: str | None = None
     ip_address: str | None = None
     anonymous_identifier: str | None = None
 
@@ -261,7 +285,26 @@ def entity_to_tool(
             inputSchema={},
             annotations=None,
         )
-    elif isinstance(entity, (Prompt, ResourceTemplate)):
+    elif isinstance(entity, ResourceTemplate):
+        # get parameters from uriTemplate
+        params = re.findall(r'\{(\w+)\}', entity.uriTemplate)
+        return Tool(
+            name=entity.name,
+            description=entity.description,
+            inputSchema={
+                "type": "object",
+                "properties": {
+                    param: {
+                        "type": "string",
+                        "description": param,
+                    }
+                    for param in params
+                },
+                "required": params,
+            },
+            annotations=None,
+        )
+    elif isinstance(entity, Prompt):
         return Tool(
             name=entity.name,
             description=entity.description,
@@ -292,3 +335,8 @@ class ToxicFlowExtraData(RootModel[dict[str, list[ToolReferenceWithLabel]]]):
 
 class AnalysisServerResponse(BaseModel):
     issues: list[Issue]
+    labels: list[list[ScalarToolLabels]]
+
+class ScanPathResultsCreate(BaseModel):
+    scan_path_results: list[ScanPathResult]
+    scan_user_info: ScanUserInfo
